using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

namespace ClipboardExample
{
   public partial class DataRowCopy : Form, IClipboardSupport
   {

      private DataTable mDataTable;

      public DataRowCopy()
      {
         InitializeComponent();
      }

      private void DataRowCopy_Load(object sender, EventArgs e)
      {
         mDataTable = new DataTable("testTable");
         mDataTable.Columns.Add("FirstName", typeof(string));
         mDataTable.Columns.Add("LastName", typeof(string));
         mDataTable.Rows.Add(mDataTable.NewRow());

         this.bindingSource.DataSource = mDataTable;
         this.textBox1.DataBindings.Add(new Binding("Text", bindingSource, "FirstName"));
         this.textBox2.DataBindings.Add(new Binding("Text", bindingSource, "LastName"));
         this.customersListBox.DataSource = this.bindingSource;
         this.customersListBox.DisplayMember = "LastName";
         this.bindingSource.MoveFirst();
      }


      #region IClipboardSupport Members

      public void CopyData()
      {

         Clipboard.SetDataObject(BuildDataObject());
      }

      public void PasteData()
      {
         FillFromDataObject(Clipboard.GetDataObject());
      }

      private DataObject BuildDataObject()
      {
         DataObject dataObject = new DataObject();
         if (this.bindingSource.Current != null)
         {
            dataObject.SetData("CustomerRowFormat", this.bindingSource.Current);
            dataObject.SetText(this.bindingSource.Current.ToString());
         }
         dataObject.SetData("CustomerTableFormat", this.mDataTable);
         return dataObject;
      }

      private void FillFromDataObject(IDataObject dataObject)
      {
         if (dataObject.GetDataPresent("CustomerRowFormat") && dataObject.GetDataPresent("CustomerTableFormat"))
         {
            // Not sure which to use, so ask customer
            RecordOrCollectionDialog dialog = new RecordOrCollectionDialog();
            if (dialog.ShowDialog() != DialogResult.Cancel)
            {
               if (dialog.Result == RecordOrCollectionResult.Record)
               {
                  DataRowView dataRowView = dataObject.GetData("CustomerRowFormat") as DataRowView;
                  if (dataRowView != null)
                  {
                     mDataTable.ImportRow(dataRowView.Row);
                     this.bindingSource.MoveLast();
                  }
               }
               else if (dialog.Result == RecordOrCollectionResult.Collection)
               {
                  DataTable dataTable = dataObject.GetData("CustomerTableFormat") as DataTable;
                  if (dataTable != null)
                  {
                     mDataTable = dataTable;
                     this.bindingSource.DataSource = mDataTable;
                     this.customersListBox.Refresh();
                  }
               }
            }
         }
      }

      #endregion

      private void textBox2_TextChanged(object sender, EventArgs e)
      {
         this.customersListBox.Refresh();
      }

      private void newButton_Click(object sender, EventArgs e)
      {
         mDataTable.Rows.Add(mDataTable.NewRow());
         this.bindingSource.MoveLast();
      }

      private void customersListBox_SelectedIndexChanged(object sender, EventArgs e)
      {
         this.bindingSource.Position = customersListBox.SelectedIndex;
      }
   }
}